# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
import mock
import packaging.version

import grpc
from grpc.experimental import aio
import math
import pytest
from proto.marshal.rules.dates import DurationRule, TimestampRule


from google.api_core import client_options
from google.api_core import exceptions as core_exceptions
from google.api_core import future
from google.api_core import gapic_v1
from google.api_core import grpc_helpers
from google.api_core import grpc_helpers_async
from google.api_core import operation_async  # type: ignore
from google.api_core import operations_v1
from google.auth import credentials as ga_credentials
from google.auth.exceptions import MutualTLSChannelError
from google.cloud.network_security_v1beta1.services.network_security import (
    NetworkSecurityAsyncClient,
)
from google.cloud.network_security_v1beta1.services.network_security import (
    NetworkSecurityClient,
)
from google.cloud.network_security_v1beta1.services.network_security import pagers
from google.cloud.network_security_v1beta1.services.network_security import transports
from google.cloud.network_security_v1beta1.services.network_security.transports.base import (
    _GOOGLE_AUTH_VERSION,
)
from google.cloud.network_security_v1beta1.types import authorization_policy
from google.cloud.network_security_v1beta1.types import (
    authorization_policy as gcn_authorization_policy,
)
from google.cloud.network_security_v1beta1.types import client_tls_policy
from google.cloud.network_security_v1beta1.types import (
    client_tls_policy as gcn_client_tls_policy,
)
from google.cloud.network_security_v1beta1.types import common
from google.cloud.network_security_v1beta1.types import server_tls_policy
from google.cloud.network_security_v1beta1.types import (
    server_tls_policy as gcn_server_tls_policy,
)
from google.cloud.network_security_v1beta1.types import tls
from google.longrunning import operations_pb2
from google.oauth2 import service_account
from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
import google.auth


# TODO(busunkim): Once google-auth >= 1.25.0 is required transitively
# through google-api-core:
# - Delete the auth "less than" test cases
# - Delete these pytest markers (Make the "greater than or equal to" tests the default).
requires_google_auth_lt_1_25_0 = pytest.mark.skipif(
    packaging.version.parse(_GOOGLE_AUTH_VERSION) >= packaging.version.parse("1.25.0"),
    reason="This test requires google-auth < 1.25.0",
)
requires_google_auth_gte_1_25_0 = pytest.mark.skipif(
    packaging.version.parse(_GOOGLE_AUTH_VERSION) < packaging.version.parse("1.25.0"),
    reason="This test requires google-auth >= 1.25.0",
)


def client_cert_source_callback():
    return b"cert bytes", b"key bytes"


# If default endpoint is localhost, then default mtls endpoint will be the same.
# This method modifies the default endpoint so the client can produce a different
# mtls endpoint for endpoint testing purposes.
def modify_default_endpoint(client):
    return (
        "foo.googleapis.com"
        if ("localhost" in client.DEFAULT_ENDPOINT)
        else client.DEFAULT_ENDPOINT
    )


def test__get_default_mtls_endpoint():
    api_endpoint = "example.googleapis.com"
    api_mtls_endpoint = "example.mtls.googleapis.com"
    sandbox_endpoint = "example.sandbox.googleapis.com"
    sandbox_mtls_endpoint = "example.mtls.sandbox.googleapis.com"
    non_googleapi = "api.example.com"

    assert NetworkSecurityClient._get_default_mtls_endpoint(None) is None
    assert (
        NetworkSecurityClient._get_default_mtls_endpoint(api_endpoint)
        == api_mtls_endpoint
    )
    assert (
        NetworkSecurityClient._get_default_mtls_endpoint(api_mtls_endpoint)
        == api_mtls_endpoint
    )
    assert (
        NetworkSecurityClient._get_default_mtls_endpoint(sandbox_endpoint)
        == sandbox_mtls_endpoint
    )
    assert (
        NetworkSecurityClient._get_default_mtls_endpoint(sandbox_mtls_endpoint)
        == sandbox_mtls_endpoint
    )
    assert (
        NetworkSecurityClient._get_default_mtls_endpoint(non_googleapi) == non_googleapi
    )


@pytest.mark.parametrize(
    "client_class", [NetworkSecurityClient, NetworkSecurityAsyncClient,]
)
def test_network_security_client_from_service_account_info(client_class):
    creds = ga_credentials.AnonymousCredentials()
    with mock.patch.object(
        service_account.Credentials, "from_service_account_info"
    ) as factory:
        factory.return_value = creds
        info = {"valid": True}
        client = client_class.from_service_account_info(info)
        assert client.transport._credentials == creds
        assert isinstance(client, client_class)

        assert client.transport._host == "networksecurity.googleapis.com:443"


@pytest.mark.parametrize(
    "transport_class,transport_name",
    [
        (transports.NetworkSecurityGrpcTransport, "grpc"),
        (transports.NetworkSecurityGrpcAsyncIOTransport, "grpc_asyncio"),
    ],
)
def test_network_security_client_service_account_always_use_jwt(
    transport_class, transport_name
):
    with mock.patch.object(
        service_account.Credentials, "with_always_use_jwt_access", create=True
    ) as use_jwt:
        creds = service_account.Credentials(None, None, None)
        transport = transport_class(credentials=creds, always_use_jwt_access=True)
        use_jwt.assert_called_once_with(True)

    with mock.patch.object(
        service_account.Credentials, "with_always_use_jwt_access", create=True
    ) as use_jwt:
        creds = service_account.Credentials(None, None, None)
        transport = transport_class(credentials=creds, always_use_jwt_access=False)
        use_jwt.assert_not_called()


@pytest.mark.parametrize(
    "client_class", [NetworkSecurityClient, NetworkSecurityAsyncClient,]
)
def test_network_security_client_from_service_account_file(client_class):
    creds = ga_credentials.AnonymousCredentials()
    with mock.patch.object(
        service_account.Credentials, "from_service_account_file"
    ) as factory:
        factory.return_value = creds
        client = client_class.from_service_account_file("dummy/file/path.json")
        assert client.transport._credentials == creds
        assert isinstance(client, client_class)

        client = client_class.from_service_account_json("dummy/file/path.json")
        assert client.transport._credentials == creds
        assert isinstance(client, client_class)

        assert client.transport._host == "networksecurity.googleapis.com:443"


def test_network_security_client_get_transport_class():
    transport = NetworkSecurityClient.get_transport_class()
    available_transports = [
        transports.NetworkSecurityGrpcTransport,
    ]
    assert transport in available_transports

    transport = NetworkSecurityClient.get_transport_class("grpc")
    assert transport == transports.NetworkSecurityGrpcTransport


@pytest.mark.parametrize(
    "client_class,transport_class,transport_name",
    [
        (NetworkSecurityClient, transports.NetworkSecurityGrpcTransport, "grpc"),
        (
            NetworkSecurityAsyncClient,
            transports.NetworkSecurityGrpcAsyncIOTransport,
            "grpc_asyncio",
        ),
    ],
)
@mock.patch.object(
    NetworkSecurityClient,
    "DEFAULT_ENDPOINT",
    modify_default_endpoint(NetworkSecurityClient),
)
@mock.patch.object(
    NetworkSecurityAsyncClient,
    "DEFAULT_ENDPOINT",
    modify_default_endpoint(NetworkSecurityAsyncClient),
)
def test_network_security_client_client_options(
    client_class, transport_class, transport_name
):
    # Check that if channel is provided we won't create a new one.
    with mock.patch.object(NetworkSecurityClient, "get_transport_class") as gtc:
        transport = transport_class(credentials=ga_credentials.AnonymousCredentials())
        client = client_class(transport=transport)
        gtc.assert_not_called()

    # Check that if channel is provided via str we will create a new one.
    with mock.patch.object(NetworkSecurityClient, "get_transport_class") as gtc:
        client = client_class(transport=transport_name)
        gtc.assert_called()

    # Check the case api_endpoint is provided.
    options = client_options.ClientOptions(api_endpoint="squid.clam.whelk")
    with mock.patch.object(transport_class, "__init__") as patched:
        patched.return_value = None
        client = client_class(client_options=options)
        patched.assert_called_once_with(
            credentials=None,
            credentials_file=None,
            host="squid.clam.whelk",
            scopes=None,
            client_cert_source_for_mtls=None,
            quota_project_id=None,
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
        )

    # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT is
    # "never".
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
        with mock.patch.object(transport_class, "__init__") as patched:
            patched.return_value = None
            client = client_class()
            patched.assert_called_once_with(
                credentials=None,
                credentials_file=None,
                host=client.DEFAULT_ENDPOINT,
                scopes=None,
                client_cert_source_for_mtls=None,
                quota_project_id=None,
                client_info=transports.base.DEFAULT_CLIENT_INFO,
                always_use_jwt_access=True,
            )

    # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT is
    # "always".
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "always"}):
        with mock.patch.object(transport_class, "__init__") as patched:
            patched.return_value = None
            client = client_class()
            patched.assert_called_once_with(
                credentials=None,
                credentials_file=None,
                host=client.DEFAULT_MTLS_ENDPOINT,
                scopes=None,
                client_cert_source_for_mtls=None,
                quota_project_id=None,
                client_info=transports.base.DEFAULT_CLIENT_INFO,
                always_use_jwt_access=True,
            )

    # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT has
    # unsupported value.
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "Unsupported"}):
        with pytest.raises(MutualTLSChannelError):
            client = client_class()

    # Check the case GOOGLE_API_USE_CLIENT_CERTIFICATE has unsupported value.
    with mock.patch.dict(
        os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "Unsupported"}
    ):
        with pytest.raises(ValueError):
            client = client_class()

    # Check the case quota_project_id is provided
    options = client_options.ClientOptions(quota_project_id="octopus")
    with mock.patch.object(transport_class, "__init__") as patched:
        patched.return_value = None
        client = client_class(client_options=options)
        patched.assert_called_once_with(
            credentials=None,
            credentials_file=None,
            host=client.DEFAULT_ENDPOINT,
            scopes=None,
            client_cert_source_for_mtls=None,
            quota_project_id="octopus",
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
        )


@pytest.mark.parametrize(
    "client_class,transport_class,transport_name,use_client_cert_env",
    [
        (
            NetworkSecurityClient,
            transports.NetworkSecurityGrpcTransport,
            "grpc",
            "true",
        ),
        (
            NetworkSecurityAsyncClient,
            transports.NetworkSecurityGrpcAsyncIOTransport,
            "grpc_asyncio",
            "true",
        ),
        (
            NetworkSecurityClient,
            transports.NetworkSecurityGrpcTransport,
            "grpc",
            "false",
        ),
        (
            NetworkSecurityAsyncClient,
            transports.NetworkSecurityGrpcAsyncIOTransport,
            "grpc_asyncio",
            "false",
        ),
    ],
)
@mock.patch.object(
    NetworkSecurityClient,
    "DEFAULT_ENDPOINT",
    modify_default_endpoint(NetworkSecurityClient),
)
@mock.patch.object(
    NetworkSecurityAsyncClient,
    "DEFAULT_ENDPOINT",
    modify_default_endpoint(NetworkSecurityAsyncClient),
)
@mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "auto"})
def test_network_security_client_mtls_env_auto(
    client_class, transport_class, transport_name, use_client_cert_env
):
    # This tests the endpoint autoswitch behavior. Endpoint is autoswitched to the default
    # mtls endpoint, if GOOGLE_API_USE_CLIENT_CERTIFICATE is "true" and client cert exists.

    # Check the case client_cert_source is provided. Whether client cert is used depends on
    # GOOGLE_API_USE_CLIENT_CERTIFICATE value.
    with mock.patch.dict(
        os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": use_client_cert_env}
    ):
        options = client_options.ClientOptions(
            client_cert_source=client_cert_source_callback
        )
        with mock.patch.object(transport_class, "__init__") as patched:
            patched.return_value = None
            client = client_class(client_options=options)

            if use_client_cert_env == "false":
                expected_client_cert_source = None
                expected_host = client.DEFAULT_ENDPOINT
            else:
                expected_client_cert_source = client_cert_source_callback
                expected_host = client.DEFAULT_MTLS_ENDPOINT

            patched.assert_called_once_with(
                credentials=None,
                credentials_file=None,
                host=expected_host,
                scopes=None,
                client_cert_source_for_mtls=expected_client_cert_source,
                quota_project_id=None,
                client_info=transports.base.DEFAULT_CLIENT_INFO,
                always_use_jwt_access=True,
            )

    # Check the case ADC client cert is provided. Whether client cert is used depends on
    # GOOGLE_API_USE_CLIENT_CERTIFICATE value.
    with mock.patch.dict(
        os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": use_client_cert_env}
    ):
        with mock.patch.object(transport_class, "__init__") as patched:
            with mock.patch(
                "google.auth.transport.mtls.has_default_client_cert_source",
                return_value=True,
            ):
                with mock.patch(
                    "google.auth.transport.mtls.default_client_cert_source",
                    return_value=client_cert_source_callback,
                ):
                    if use_client_cert_env == "false":
                        expected_host = client.DEFAULT_ENDPOINT
                        expected_client_cert_source = None
                    else:
                        expected_host = client.DEFAULT_MTLS_ENDPOINT
                        expected_client_cert_source = client_cert_source_callback

                    patched.return_value = None
                    client = client_class()
                    patched.assert_called_once_with(
                        credentials=None,
                        credentials_file=None,
                        host=expected_host,
                        scopes=None,
                        client_cert_source_for_mtls=expected_client_cert_source,
                        quota_project_id=None,
                        client_info=transports.base.DEFAULT_CLIENT_INFO,
                        always_use_jwt_access=True,
                    )

    # Check the case client_cert_source and ADC client cert are not provided.
    with mock.patch.dict(
        os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": use_client_cert_env}
    ):
        with mock.patch.object(transport_class, "__init__") as patched:
            with mock.patch(
                "google.auth.transport.mtls.has_default_client_cert_source",
                return_value=False,
            ):
                patched.return_value = None
                client = client_class()
                patched.assert_called_once_with(
                    credentials=None,
                    credentials_file=None,
                    host=client.DEFAULT_ENDPOINT,
                    scopes=None,
                    client_cert_source_for_mtls=None,
                    quota_project_id=None,
                    client_info=transports.base.DEFAULT_CLIENT_INFO,
                    always_use_jwt_access=True,
                )


@pytest.mark.parametrize(
    "client_class,transport_class,transport_name",
    [
        (NetworkSecurityClient, transports.NetworkSecurityGrpcTransport, "grpc"),
        (
            NetworkSecurityAsyncClient,
            transports.NetworkSecurityGrpcAsyncIOTransport,
            "grpc_asyncio",
        ),
    ],
)
def test_network_security_client_client_options_scopes(
    client_class, transport_class, transport_name
):
    # Check the case scopes are provided.
    options = client_options.ClientOptions(scopes=["1", "2"],)
    with mock.patch.object(transport_class, "__init__") as patched:
        patched.return_value = None
        client = client_class(client_options=options)
        patched.assert_called_once_with(
            credentials=None,
            credentials_file=None,
            host=client.DEFAULT_ENDPOINT,
            scopes=["1", "2"],
            client_cert_source_for_mtls=None,
            quota_project_id=None,
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
        )


@pytest.mark.parametrize(
    "client_class,transport_class,transport_name",
    [
        (NetworkSecurityClient, transports.NetworkSecurityGrpcTransport, "grpc"),
        (
            NetworkSecurityAsyncClient,
            transports.NetworkSecurityGrpcAsyncIOTransport,
            "grpc_asyncio",
        ),
    ],
)
def test_network_security_client_client_options_credentials_file(
    client_class, transport_class, transport_name
):
    # Check the case credentials file is provided.
    options = client_options.ClientOptions(credentials_file="credentials.json")
    with mock.patch.object(transport_class, "__init__") as patched:
        patched.return_value = None
        client = client_class(client_options=options)
        patched.assert_called_once_with(
            credentials=None,
            credentials_file="credentials.json",
            host=client.DEFAULT_ENDPOINT,
            scopes=None,
            client_cert_source_for_mtls=None,
            quota_project_id=None,
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
        )


def test_network_security_client_client_options_from_dict():
    with mock.patch(
        "google.cloud.network_security_v1beta1.services.network_security.transports.NetworkSecurityGrpcTransport.__init__"
    ) as grpc_transport:
        grpc_transport.return_value = None
        client = NetworkSecurityClient(
            client_options={"api_endpoint": "squid.clam.whelk"}
        )
        grpc_transport.assert_called_once_with(
            credentials=None,
            credentials_file=None,
            host="squid.clam.whelk",
            scopes=None,
            client_cert_source_for_mtls=None,
            quota_project_id=None,
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
        )


def test_list_authorization_policies(
    transport: str = "grpc",
    request_type=authorization_policy.ListAuthorizationPoliciesRequest,
):
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_authorization_policies), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = authorization_policy.ListAuthorizationPoliciesResponse(
            next_page_token="next_page_token_value",
        )
        response = client.list_authorization_policies(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == authorization_policy.ListAuthorizationPoliciesRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, pagers.ListAuthorizationPoliciesPager)
    assert response.next_page_token == "next_page_token_value"


def test_list_authorization_policies_from_dict():
    test_list_authorization_policies(request_type=dict)


def test_list_authorization_policies_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_authorization_policies), "__call__"
    ) as call:
        client.list_authorization_policies()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == authorization_policy.ListAuthorizationPoliciesRequest()


@pytest.mark.asyncio
async def test_list_authorization_policies_async(
    transport: str = "grpc_asyncio",
    request_type=authorization_policy.ListAuthorizationPoliciesRequest,
):
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_authorization_policies), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            authorization_policy.ListAuthorizationPoliciesResponse(
                next_page_token="next_page_token_value",
            )
        )
        response = await client.list_authorization_policies(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == authorization_policy.ListAuthorizationPoliciesRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, pagers.ListAuthorizationPoliciesAsyncPager)
    assert response.next_page_token == "next_page_token_value"


@pytest.mark.asyncio
async def test_list_authorization_policies_async_from_dict():
    await test_list_authorization_policies_async(request_type=dict)


def test_list_authorization_policies_field_headers():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = authorization_policy.ListAuthorizationPoliciesRequest()

    request.parent = "parent/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_authorization_policies), "__call__"
    ) as call:
        call.return_value = authorization_policy.ListAuthorizationPoliciesResponse()
        client.list_authorization_policies(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "parent=parent/value",) in kw["metadata"]


@pytest.mark.asyncio
async def test_list_authorization_policies_field_headers_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = authorization_policy.ListAuthorizationPoliciesRequest()

    request.parent = "parent/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_authorization_policies), "__call__"
    ) as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            authorization_policy.ListAuthorizationPoliciesResponse()
        )
        await client.list_authorization_policies(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "parent=parent/value",) in kw["metadata"]


def test_list_authorization_policies_flattened():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_authorization_policies), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = authorization_policy.ListAuthorizationPoliciesResponse()
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.list_authorization_policies(parent="parent_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0].parent == "parent_value"


def test_list_authorization_policies_flattened_error():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.list_authorization_policies(
            authorization_policy.ListAuthorizationPoliciesRequest(),
            parent="parent_value",
        )


@pytest.mark.asyncio
async def test_list_authorization_policies_flattened_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_authorization_policies), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = authorization_policy.ListAuthorizationPoliciesResponse()

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            authorization_policy.ListAuthorizationPoliciesResponse()
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.list_authorization_policies(parent="parent_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0].parent == "parent_value"


@pytest.mark.asyncio
async def test_list_authorization_policies_flattened_error_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.list_authorization_policies(
            authorization_policy.ListAuthorizationPoliciesRequest(),
            parent="parent_value",
        )


def test_list_authorization_policies_pager():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials,)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_authorization_policies), "__call__"
    ) as call:
        # Set the response to a series of pages.
        call.side_effect = (
            authorization_policy.ListAuthorizationPoliciesResponse(
                authorization_policies=[
                    authorization_policy.AuthorizationPolicy(),
                    authorization_policy.AuthorizationPolicy(),
                    authorization_policy.AuthorizationPolicy(),
                ],
                next_page_token="abc",
            ),
            authorization_policy.ListAuthorizationPoliciesResponse(
                authorization_policies=[], next_page_token="def",
            ),
            authorization_policy.ListAuthorizationPoliciesResponse(
                authorization_policies=[authorization_policy.AuthorizationPolicy(),],
                next_page_token="ghi",
            ),
            authorization_policy.ListAuthorizationPoliciesResponse(
                authorization_policies=[
                    authorization_policy.AuthorizationPolicy(),
                    authorization_policy.AuthorizationPolicy(),
                ],
            ),
            RuntimeError,
        )

        metadata = ()
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", ""),)),
        )
        pager = client.list_authorization_policies(request={})

        assert pager._metadata == metadata

        results = [i for i in pager]
        assert len(results) == 6
        assert all(
            isinstance(i, authorization_policy.AuthorizationPolicy) for i in results
        )


def test_list_authorization_policies_pages():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials,)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_authorization_policies), "__call__"
    ) as call:
        # Set the response to a series of pages.
        call.side_effect = (
            authorization_policy.ListAuthorizationPoliciesResponse(
                authorization_policies=[
                    authorization_policy.AuthorizationPolicy(),
                    authorization_policy.AuthorizationPolicy(),
                    authorization_policy.AuthorizationPolicy(),
                ],
                next_page_token="abc",
            ),
            authorization_policy.ListAuthorizationPoliciesResponse(
                authorization_policies=[], next_page_token="def",
            ),
            authorization_policy.ListAuthorizationPoliciesResponse(
                authorization_policies=[authorization_policy.AuthorizationPolicy(),],
                next_page_token="ghi",
            ),
            authorization_policy.ListAuthorizationPoliciesResponse(
                authorization_policies=[
                    authorization_policy.AuthorizationPolicy(),
                    authorization_policy.AuthorizationPolicy(),
                ],
            ),
            RuntimeError,
        )
        pages = list(client.list_authorization_policies(request={}).pages)
        for page_, token in zip(pages, ["abc", "def", "ghi", ""]):
            assert page_.raw_page.next_page_token == token


@pytest.mark.asyncio
async def test_list_authorization_policies_async_pager():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials,
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_authorization_policies),
        "__call__",
        new_callable=mock.AsyncMock,
    ) as call:
        # Set the response to a series of pages.
        call.side_effect = (
            authorization_policy.ListAuthorizationPoliciesResponse(
                authorization_policies=[
                    authorization_policy.AuthorizationPolicy(),
                    authorization_policy.AuthorizationPolicy(),
                    authorization_policy.AuthorizationPolicy(),
                ],
                next_page_token="abc",
            ),
            authorization_policy.ListAuthorizationPoliciesResponse(
                authorization_policies=[], next_page_token="def",
            ),
            authorization_policy.ListAuthorizationPoliciesResponse(
                authorization_policies=[authorization_policy.AuthorizationPolicy(),],
                next_page_token="ghi",
            ),
            authorization_policy.ListAuthorizationPoliciesResponse(
                authorization_policies=[
                    authorization_policy.AuthorizationPolicy(),
                    authorization_policy.AuthorizationPolicy(),
                ],
            ),
            RuntimeError,
        )
        async_pager = await client.list_authorization_policies(request={},)
        assert async_pager.next_page_token == "abc"
        responses = []
        async for response in async_pager:
            responses.append(response)

        assert len(responses) == 6
        assert all(
            isinstance(i, authorization_policy.AuthorizationPolicy) for i in responses
        )


@pytest.mark.asyncio
async def test_list_authorization_policies_async_pages():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials,
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_authorization_policies),
        "__call__",
        new_callable=mock.AsyncMock,
    ) as call:
        # Set the response to a series of pages.
        call.side_effect = (
            authorization_policy.ListAuthorizationPoliciesResponse(
                authorization_policies=[
                    authorization_policy.AuthorizationPolicy(),
                    authorization_policy.AuthorizationPolicy(),
                    authorization_policy.AuthorizationPolicy(),
                ],
                next_page_token="abc",
            ),
            authorization_policy.ListAuthorizationPoliciesResponse(
                authorization_policies=[], next_page_token="def",
            ),
            authorization_policy.ListAuthorizationPoliciesResponse(
                authorization_policies=[authorization_policy.AuthorizationPolicy(),],
                next_page_token="ghi",
            ),
            authorization_policy.ListAuthorizationPoliciesResponse(
                authorization_policies=[
                    authorization_policy.AuthorizationPolicy(),
                    authorization_policy.AuthorizationPolicy(),
                ],
            ),
            RuntimeError,
        )
        pages = []
        async for page_ in (await client.list_authorization_policies(request={})).pages:
            pages.append(page_)
        for page_, token in zip(pages, ["abc", "def", "ghi", ""]):
            assert page_.raw_page.next_page_token == token


def test_get_authorization_policy(
    transport: str = "grpc",
    request_type=authorization_policy.GetAuthorizationPolicyRequest,
):
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_authorization_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = authorization_policy.AuthorizationPolicy(
            name="name_value",
            description="description_value",
            action=authorization_policy.AuthorizationPolicy.Action.ALLOW,
        )
        response = client.get_authorization_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == authorization_policy.GetAuthorizationPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, authorization_policy.AuthorizationPolicy)
    assert response.name == "name_value"
    assert response.description == "description_value"
    assert response.action == authorization_policy.AuthorizationPolicy.Action.ALLOW


def test_get_authorization_policy_from_dict():
    test_get_authorization_policy(request_type=dict)


def test_get_authorization_policy_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_authorization_policy), "__call__"
    ) as call:
        client.get_authorization_policy()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == authorization_policy.GetAuthorizationPolicyRequest()


@pytest.mark.asyncio
async def test_get_authorization_policy_async(
    transport: str = "grpc_asyncio",
    request_type=authorization_policy.GetAuthorizationPolicyRequest,
):
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_authorization_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            authorization_policy.AuthorizationPolicy(
                name="name_value",
                description="description_value",
                action=authorization_policy.AuthorizationPolicy.Action.ALLOW,
            )
        )
        response = await client.get_authorization_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == authorization_policy.GetAuthorizationPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, authorization_policy.AuthorizationPolicy)
    assert response.name == "name_value"
    assert response.description == "description_value"
    assert response.action == authorization_policy.AuthorizationPolicy.Action.ALLOW


@pytest.mark.asyncio
async def test_get_authorization_policy_async_from_dict():
    await test_get_authorization_policy_async(request_type=dict)


def test_get_authorization_policy_field_headers():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = authorization_policy.GetAuthorizationPolicyRequest()

    request.name = "name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_authorization_policy), "__call__"
    ) as call:
        call.return_value = authorization_policy.AuthorizationPolicy()
        client.get_authorization_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]


@pytest.mark.asyncio
async def test_get_authorization_policy_field_headers_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = authorization_policy.GetAuthorizationPolicyRequest()

    request.name = "name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_authorization_policy), "__call__"
    ) as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            authorization_policy.AuthorizationPolicy()
        )
        await client.get_authorization_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]


def test_get_authorization_policy_flattened():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_authorization_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = authorization_policy.AuthorizationPolicy()
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.get_authorization_policy(name="name_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0].name == "name_value"


def test_get_authorization_policy_flattened_error():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.get_authorization_policy(
            authorization_policy.GetAuthorizationPolicyRequest(), name="name_value",
        )


@pytest.mark.asyncio
async def test_get_authorization_policy_flattened_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_authorization_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = authorization_policy.AuthorizationPolicy()

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            authorization_policy.AuthorizationPolicy()
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.get_authorization_policy(name="name_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0].name == "name_value"


@pytest.mark.asyncio
async def test_get_authorization_policy_flattened_error_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.get_authorization_policy(
            authorization_policy.GetAuthorizationPolicyRequest(), name="name_value",
        )


def test_create_authorization_policy(
    transport: str = "grpc",
    request_type=gcn_authorization_policy.CreateAuthorizationPolicyRequest,
):
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_authorization_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/spam")
        response = client.create_authorization_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_authorization_policy.CreateAuthorizationPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


def test_create_authorization_policy_from_dict():
    test_create_authorization_policy(request_type=dict)


def test_create_authorization_policy_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_authorization_policy), "__call__"
    ) as call:
        client.create_authorization_policy()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_authorization_policy.CreateAuthorizationPolicyRequest()


@pytest.mark.asyncio
async def test_create_authorization_policy_async(
    transport: str = "grpc_asyncio",
    request_type=gcn_authorization_policy.CreateAuthorizationPolicyRequest,
):
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_authorization_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        response = await client.create_authorization_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_authorization_policy.CreateAuthorizationPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


@pytest.mark.asyncio
async def test_create_authorization_policy_async_from_dict():
    await test_create_authorization_policy_async(request_type=dict)


def test_create_authorization_policy_field_headers():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = gcn_authorization_policy.CreateAuthorizationPolicyRequest()

    request.parent = "parent/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_authorization_policy), "__call__"
    ) as call:
        call.return_value = operations_pb2.Operation(name="operations/op")
        client.create_authorization_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "parent=parent/value",) in kw["metadata"]


@pytest.mark.asyncio
async def test_create_authorization_policy_field_headers_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = gcn_authorization_policy.CreateAuthorizationPolicyRequest()

    request.parent = "parent/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_authorization_policy), "__call__"
    ) as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/op")
        )
        await client.create_authorization_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "parent=parent/value",) in kw["metadata"]


def test_create_authorization_policy_flattened():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_authorization_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.create_authorization_policy(
            parent="parent_value",
            authorization_policy=gcn_authorization_policy.AuthorizationPolicy(
                name="name_value"
            ),
            authorization_policy_id="authorization_policy_id_value",
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0].parent == "parent_value"
        assert args[
            0
        ].authorization_policy == gcn_authorization_policy.AuthorizationPolicy(
            name="name_value"
        )
        assert args[0].authorization_policy_id == "authorization_policy_id_value"


def test_create_authorization_policy_flattened_error():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.create_authorization_policy(
            gcn_authorization_policy.CreateAuthorizationPolicyRequest(),
            parent="parent_value",
            authorization_policy=gcn_authorization_policy.AuthorizationPolicy(
                name="name_value"
            ),
            authorization_policy_id="authorization_policy_id_value",
        )


@pytest.mark.asyncio
async def test_create_authorization_policy_flattened_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_authorization_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.create_authorization_policy(
            parent="parent_value",
            authorization_policy=gcn_authorization_policy.AuthorizationPolicy(
                name="name_value"
            ),
            authorization_policy_id="authorization_policy_id_value",
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0].parent == "parent_value"
        assert args[
            0
        ].authorization_policy == gcn_authorization_policy.AuthorizationPolicy(
            name="name_value"
        )
        assert args[0].authorization_policy_id == "authorization_policy_id_value"


@pytest.mark.asyncio
async def test_create_authorization_policy_flattened_error_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.create_authorization_policy(
            gcn_authorization_policy.CreateAuthorizationPolicyRequest(),
            parent="parent_value",
            authorization_policy=gcn_authorization_policy.AuthorizationPolicy(
                name="name_value"
            ),
            authorization_policy_id="authorization_policy_id_value",
        )


def test_update_authorization_policy(
    transport: str = "grpc",
    request_type=gcn_authorization_policy.UpdateAuthorizationPolicyRequest,
):
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_authorization_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/spam")
        response = client.update_authorization_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_authorization_policy.UpdateAuthorizationPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


def test_update_authorization_policy_from_dict():
    test_update_authorization_policy(request_type=dict)


def test_update_authorization_policy_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_authorization_policy), "__call__"
    ) as call:
        client.update_authorization_policy()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_authorization_policy.UpdateAuthorizationPolicyRequest()


@pytest.mark.asyncio
async def test_update_authorization_policy_async(
    transport: str = "grpc_asyncio",
    request_type=gcn_authorization_policy.UpdateAuthorizationPolicyRequest,
):
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_authorization_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        response = await client.update_authorization_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_authorization_policy.UpdateAuthorizationPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


@pytest.mark.asyncio
async def test_update_authorization_policy_async_from_dict():
    await test_update_authorization_policy_async(request_type=dict)


def test_update_authorization_policy_field_headers():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = gcn_authorization_policy.UpdateAuthorizationPolicyRequest()

    request.authorization_policy.name = "authorization_policy.name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_authorization_policy), "__call__"
    ) as call:
        call.return_value = operations_pb2.Operation(name="operations/op")
        client.update_authorization_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert (
        "x-goog-request-params",
        "authorization_policy.name=authorization_policy.name/value",
    ) in kw["metadata"]


@pytest.mark.asyncio
async def test_update_authorization_policy_field_headers_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = gcn_authorization_policy.UpdateAuthorizationPolicyRequest()

    request.authorization_policy.name = "authorization_policy.name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_authorization_policy), "__call__"
    ) as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/op")
        )
        await client.update_authorization_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert (
        "x-goog-request-params",
        "authorization_policy.name=authorization_policy.name/value",
    ) in kw["metadata"]


def test_update_authorization_policy_flattened():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_authorization_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.update_authorization_policy(
            authorization_policy=gcn_authorization_policy.AuthorizationPolicy(
                name="name_value"
            ),
            update_mask=field_mask_pb2.FieldMask(paths=["paths_value"]),
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[
            0
        ].authorization_policy == gcn_authorization_policy.AuthorizationPolicy(
            name="name_value"
        )
        assert args[0].update_mask == field_mask_pb2.FieldMask(paths=["paths_value"])


def test_update_authorization_policy_flattened_error():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.update_authorization_policy(
            gcn_authorization_policy.UpdateAuthorizationPolicyRequest(),
            authorization_policy=gcn_authorization_policy.AuthorizationPolicy(
                name="name_value"
            ),
            update_mask=field_mask_pb2.FieldMask(paths=["paths_value"]),
        )


@pytest.mark.asyncio
async def test_update_authorization_policy_flattened_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_authorization_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.update_authorization_policy(
            authorization_policy=gcn_authorization_policy.AuthorizationPolicy(
                name="name_value"
            ),
            update_mask=field_mask_pb2.FieldMask(paths=["paths_value"]),
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[
            0
        ].authorization_policy == gcn_authorization_policy.AuthorizationPolicy(
            name="name_value"
        )
        assert args[0].update_mask == field_mask_pb2.FieldMask(paths=["paths_value"])


@pytest.mark.asyncio
async def test_update_authorization_policy_flattened_error_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.update_authorization_policy(
            gcn_authorization_policy.UpdateAuthorizationPolicyRequest(),
            authorization_policy=gcn_authorization_policy.AuthorizationPolicy(
                name="name_value"
            ),
            update_mask=field_mask_pb2.FieldMask(paths=["paths_value"]),
        )


def test_delete_authorization_policy(
    transport: str = "grpc",
    request_type=authorization_policy.DeleteAuthorizationPolicyRequest,
):
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_authorization_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/spam")
        response = client.delete_authorization_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == authorization_policy.DeleteAuthorizationPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


def test_delete_authorization_policy_from_dict():
    test_delete_authorization_policy(request_type=dict)


def test_delete_authorization_policy_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_authorization_policy), "__call__"
    ) as call:
        client.delete_authorization_policy()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == authorization_policy.DeleteAuthorizationPolicyRequest()


@pytest.mark.asyncio
async def test_delete_authorization_policy_async(
    transport: str = "grpc_asyncio",
    request_type=authorization_policy.DeleteAuthorizationPolicyRequest,
):
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_authorization_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        response = await client.delete_authorization_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == authorization_policy.DeleteAuthorizationPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


@pytest.mark.asyncio
async def test_delete_authorization_policy_async_from_dict():
    await test_delete_authorization_policy_async(request_type=dict)


def test_delete_authorization_policy_field_headers():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = authorization_policy.DeleteAuthorizationPolicyRequest()

    request.name = "name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_authorization_policy), "__call__"
    ) as call:
        call.return_value = operations_pb2.Operation(name="operations/op")
        client.delete_authorization_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]


@pytest.mark.asyncio
async def test_delete_authorization_policy_field_headers_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = authorization_policy.DeleteAuthorizationPolicyRequest()

    request.name = "name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_authorization_policy), "__call__"
    ) as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/op")
        )
        await client.delete_authorization_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]


def test_delete_authorization_policy_flattened():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_authorization_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.delete_authorization_policy(name="name_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0].name == "name_value"


def test_delete_authorization_policy_flattened_error():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.delete_authorization_policy(
            authorization_policy.DeleteAuthorizationPolicyRequest(), name="name_value",
        )


@pytest.mark.asyncio
async def test_delete_authorization_policy_flattened_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_authorization_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.delete_authorization_policy(name="name_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0].name == "name_value"


@pytest.mark.asyncio
async def test_delete_authorization_policy_flattened_error_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.delete_authorization_policy(
            authorization_policy.DeleteAuthorizationPolicyRequest(), name="name_value",
        )


def test_list_server_tls_policies(
    transport: str = "grpc", request_type=server_tls_policy.ListServerTlsPoliciesRequest
):
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_server_tls_policies), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = server_tls_policy.ListServerTlsPoliciesResponse(
            next_page_token="next_page_token_value",
        )
        response = client.list_server_tls_policies(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == server_tls_policy.ListServerTlsPoliciesRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, pagers.ListServerTlsPoliciesPager)
    assert response.next_page_token == "next_page_token_value"


def test_list_server_tls_policies_from_dict():
    test_list_server_tls_policies(request_type=dict)


def test_list_server_tls_policies_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_server_tls_policies), "__call__"
    ) as call:
        client.list_server_tls_policies()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == server_tls_policy.ListServerTlsPoliciesRequest()


@pytest.mark.asyncio
async def test_list_server_tls_policies_async(
    transport: str = "grpc_asyncio",
    request_type=server_tls_policy.ListServerTlsPoliciesRequest,
):
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_server_tls_policies), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            server_tls_policy.ListServerTlsPoliciesResponse(
                next_page_token="next_page_token_value",
            )
        )
        response = await client.list_server_tls_policies(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == server_tls_policy.ListServerTlsPoliciesRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, pagers.ListServerTlsPoliciesAsyncPager)
    assert response.next_page_token == "next_page_token_value"


@pytest.mark.asyncio
async def test_list_server_tls_policies_async_from_dict():
    await test_list_server_tls_policies_async(request_type=dict)


def test_list_server_tls_policies_field_headers():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = server_tls_policy.ListServerTlsPoliciesRequest()

    request.parent = "parent/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_server_tls_policies), "__call__"
    ) as call:
        call.return_value = server_tls_policy.ListServerTlsPoliciesResponse()
        client.list_server_tls_policies(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "parent=parent/value",) in kw["metadata"]


@pytest.mark.asyncio
async def test_list_server_tls_policies_field_headers_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = server_tls_policy.ListServerTlsPoliciesRequest()

    request.parent = "parent/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_server_tls_policies), "__call__"
    ) as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            server_tls_policy.ListServerTlsPoliciesResponse()
        )
        await client.list_server_tls_policies(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "parent=parent/value",) in kw["metadata"]


def test_list_server_tls_policies_flattened():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_server_tls_policies), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = server_tls_policy.ListServerTlsPoliciesResponse()
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.list_server_tls_policies(parent="parent_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0].parent == "parent_value"


def test_list_server_tls_policies_flattened_error():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.list_server_tls_policies(
            server_tls_policy.ListServerTlsPoliciesRequest(), parent="parent_value",
        )


@pytest.mark.asyncio
async def test_list_server_tls_policies_flattened_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_server_tls_policies), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = server_tls_policy.ListServerTlsPoliciesResponse()

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            server_tls_policy.ListServerTlsPoliciesResponse()
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.list_server_tls_policies(parent="parent_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0].parent == "parent_value"


@pytest.mark.asyncio
async def test_list_server_tls_policies_flattened_error_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.list_server_tls_policies(
            server_tls_policy.ListServerTlsPoliciesRequest(), parent="parent_value",
        )


def test_list_server_tls_policies_pager():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials,)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_server_tls_policies), "__call__"
    ) as call:
        # Set the response to a series of pages.
        call.side_effect = (
            server_tls_policy.ListServerTlsPoliciesResponse(
                server_tls_policies=[
                    server_tls_policy.ServerTlsPolicy(),
                    server_tls_policy.ServerTlsPolicy(),
                    server_tls_policy.ServerTlsPolicy(),
                ],
                next_page_token="abc",
            ),
            server_tls_policy.ListServerTlsPoliciesResponse(
                server_tls_policies=[], next_page_token="def",
            ),
            server_tls_policy.ListServerTlsPoliciesResponse(
                server_tls_policies=[server_tls_policy.ServerTlsPolicy(),],
                next_page_token="ghi",
            ),
            server_tls_policy.ListServerTlsPoliciesResponse(
                server_tls_policies=[
                    server_tls_policy.ServerTlsPolicy(),
                    server_tls_policy.ServerTlsPolicy(),
                ],
            ),
            RuntimeError,
        )

        metadata = ()
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", ""),)),
        )
        pager = client.list_server_tls_policies(request={})

        assert pager._metadata == metadata

        results = [i for i in pager]
        assert len(results) == 6
        assert all(isinstance(i, server_tls_policy.ServerTlsPolicy) for i in results)


def test_list_server_tls_policies_pages():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials,)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_server_tls_policies), "__call__"
    ) as call:
        # Set the response to a series of pages.
        call.side_effect = (
            server_tls_policy.ListServerTlsPoliciesResponse(
                server_tls_policies=[
                    server_tls_policy.ServerTlsPolicy(),
                    server_tls_policy.ServerTlsPolicy(),
                    server_tls_policy.ServerTlsPolicy(),
                ],
                next_page_token="abc",
            ),
            server_tls_policy.ListServerTlsPoliciesResponse(
                server_tls_policies=[], next_page_token="def",
            ),
            server_tls_policy.ListServerTlsPoliciesResponse(
                server_tls_policies=[server_tls_policy.ServerTlsPolicy(),],
                next_page_token="ghi",
            ),
            server_tls_policy.ListServerTlsPoliciesResponse(
                server_tls_policies=[
                    server_tls_policy.ServerTlsPolicy(),
                    server_tls_policy.ServerTlsPolicy(),
                ],
            ),
            RuntimeError,
        )
        pages = list(client.list_server_tls_policies(request={}).pages)
        for page_, token in zip(pages, ["abc", "def", "ghi", ""]):
            assert page_.raw_page.next_page_token == token


@pytest.mark.asyncio
async def test_list_server_tls_policies_async_pager():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials,
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_server_tls_policies),
        "__call__",
        new_callable=mock.AsyncMock,
    ) as call:
        # Set the response to a series of pages.
        call.side_effect = (
            server_tls_policy.ListServerTlsPoliciesResponse(
                server_tls_policies=[
                    server_tls_policy.ServerTlsPolicy(),
                    server_tls_policy.ServerTlsPolicy(),
                    server_tls_policy.ServerTlsPolicy(),
                ],
                next_page_token="abc",
            ),
            server_tls_policy.ListServerTlsPoliciesResponse(
                server_tls_policies=[], next_page_token="def",
            ),
            server_tls_policy.ListServerTlsPoliciesResponse(
                server_tls_policies=[server_tls_policy.ServerTlsPolicy(),],
                next_page_token="ghi",
            ),
            server_tls_policy.ListServerTlsPoliciesResponse(
                server_tls_policies=[
                    server_tls_policy.ServerTlsPolicy(),
                    server_tls_policy.ServerTlsPolicy(),
                ],
            ),
            RuntimeError,
        )
        async_pager = await client.list_server_tls_policies(request={},)
        assert async_pager.next_page_token == "abc"
        responses = []
        async for response in async_pager:
            responses.append(response)

        assert len(responses) == 6
        assert all(isinstance(i, server_tls_policy.ServerTlsPolicy) for i in responses)


@pytest.mark.asyncio
async def test_list_server_tls_policies_async_pages():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials,
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_server_tls_policies),
        "__call__",
        new_callable=mock.AsyncMock,
    ) as call:
        # Set the response to a series of pages.
        call.side_effect = (
            server_tls_policy.ListServerTlsPoliciesResponse(
                server_tls_policies=[
                    server_tls_policy.ServerTlsPolicy(),
                    server_tls_policy.ServerTlsPolicy(),
                    server_tls_policy.ServerTlsPolicy(),
                ],
                next_page_token="abc",
            ),
            server_tls_policy.ListServerTlsPoliciesResponse(
                server_tls_policies=[], next_page_token="def",
            ),
            server_tls_policy.ListServerTlsPoliciesResponse(
                server_tls_policies=[server_tls_policy.ServerTlsPolicy(),],
                next_page_token="ghi",
            ),
            server_tls_policy.ListServerTlsPoliciesResponse(
                server_tls_policies=[
                    server_tls_policy.ServerTlsPolicy(),
                    server_tls_policy.ServerTlsPolicy(),
                ],
            ),
            RuntimeError,
        )
        pages = []
        async for page_ in (await client.list_server_tls_policies(request={})).pages:
            pages.append(page_)
        for page_, token in zip(pages, ["abc", "def", "ghi", ""]):
            assert page_.raw_page.next_page_token == token


def test_get_server_tls_policy(
    transport: str = "grpc", request_type=server_tls_policy.GetServerTlsPolicyRequest
):
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_server_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = server_tls_policy.ServerTlsPolicy(
            name="name_value", description="description_value", allow_open=True,
        )
        response = client.get_server_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == server_tls_policy.GetServerTlsPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, server_tls_policy.ServerTlsPolicy)
    assert response.name == "name_value"
    assert response.description == "description_value"
    assert response.allow_open is True


def test_get_server_tls_policy_from_dict():
    test_get_server_tls_policy(request_type=dict)


def test_get_server_tls_policy_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_server_tls_policy), "__call__"
    ) as call:
        client.get_server_tls_policy()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == server_tls_policy.GetServerTlsPolicyRequest()


@pytest.mark.asyncio
async def test_get_server_tls_policy_async(
    transport: str = "grpc_asyncio",
    request_type=server_tls_policy.GetServerTlsPolicyRequest,
):
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_server_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            server_tls_policy.ServerTlsPolicy(
                name="name_value", description="description_value", allow_open=True,
            )
        )
        response = await client.get_server_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == server_tls_policy.GetServerTlsPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, server_tls_policy.ServerTlsPolicy)
    assert response.name == "name_value"
    assert response.description == "description_value"
    assert response.allow_open is True


@pytest.mark.asyncio
async def test_get_server_tls_policy_async_from_dict():
    await test_get_server_tls_policy_async(request_type=dict)


def test_get_server_tls_policy_field_headers():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = server_tls_policy.GetServerTlsPolicyRequest()

    request.name = "name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_server_tls_policy), "__call__"
    ) as call:
        call.return_value = server_tls_policy.ServerTlsPolicy()
        client.get_server_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]


@pytest.mark.asyncio
async def test_get_server_tls_policy_field_headers_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = server_tls_policy.GetServerTlsPolicyRequest()

    request.name = "name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_server_tls_policy), "__call__"
    ) as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            server_tls_policy.ServerTlsPolicy()
        )
        await client.get_server_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]


def test_get_server_tls_policy_flattened():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_server_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = server_tls_policy.ServerTlsPolicy()
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.get_server_tls_policy(name="name_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0].name == "name_value"


def test_get_server_tls_policy_flattened_error():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.get_server_tls_policy(
            server_tls_policy.GetServerTlsPolicyRequest(), name="name_value",
        )


@pytest.mark.asyncio
async def test_get_server_tls_policy_flattened_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_server_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = server_tls_policy.ServerTlsPolicy()

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            server_tls_policy.ServerTlsPolicy()
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.get_server_tls_policy(name="name_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0].name == "name_value"


@pytest.mark.asyncio
async def test_get_server_tls_policy_flattened_error_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.get_server_tls_policy(
            server_tls_policy.GetServerTlsPolicyRequest(), name="name_value",
        )


def test_create_server_tls_policy(
    transport: str = "grpc",
    request_type=gcn_server_tls_policy.CreateServerTlsPolicyRequest,
):
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_server_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/spam")
        response = client.create_server_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_server_tls_policy.CreateServerTlsPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


def test_create_server_tls_policy_from_dict():
    test_create_server_tls_policy(request_type=dict)


def test_create_server_tls_policy_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_server_tls_policy), "__call__"
    ) as call:
        client.create_server_tls_policy()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_server_tls_policy.CreateServerTlsPolicyRequest()


@pytest.mark.asyncio
async def test_create_server_tls_policy_async(
    transport: str = "grpc_asyncio",
    request_type=gcn_server_tls_policy.CreateServerTlsPolicyRequest,
):
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_server_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        response = await client.create_server_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_server_tls_policy.CreateServerTlsPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


@pytest.mark.asyncio
async def test_create_server_tls_policy_async_from_dict():
    await test_create_server_tls_policy_async(request_type=dict)


def test_create_server_tls_policy_field_headers():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = gcn_server_tls_policy.CreateServerTlsPolicyRequest()

    request.parent = "parent/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_server_tls_policy), "__call__"
    ) as call:
        call.return_value = operations_pb2.Operation(name="operations/op")
        client.create_server_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "parent=parent/value",) in kw["metadata"]


@pytest.mark.asyncio
async def test_create_server_tls_policy_field_headers_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = gcn_server_tls_policy.CreateServerTlsPolicyRequest()

    request.parent = "parent/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_server_tls_policy), "__call__"
    ) as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/op")
        )
        await client.create_server_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "parent=parent/value",) in kw["metadata"]


def test_create_server_tls_policy_flattened():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_server_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.create_server_tls_policy(
            parent="parent_value",
            server_tls_policy=gcn_server_tls_policy.ServerTlsPolicy(name="name_value"),
            server_tls_policy_id="server_tls_policy_id_value",
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0].parent == "parent_value"
        assert args[0].server_tls_policy == gcn_server_tls_policy.ServerTlsPolicy(
            name="name_value"
        )
        assert args[0].server_tls_policy_id == "server_tls_policy_id_value"


def test_create_server_tls_policy_flattened_error():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.create_server_tls_policy(
            gcn_server_tls_policy.CreateServerTlsPolicyRequest(),
            parent="parent_value",
            server_tls_policy=gcn_server_tls_policy.ServerTlsPolicy(name="name_value"),
            server_tls_policy_id="server_tls_policy_id_value",
        )


@pytest.mark.asyncio
async def test_create_server_tls_policy_flattened_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_server_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.create_server_tls_policy(
            parent="parent_value",
            server_tls_policy=gcn_server_tls_policy.ServerTlsPolicy(name="name_value"),
            server_tls_policy_id="server_tls_policy_id_value",
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0].parent == "parent_value"
        assert args[0].server_tls_policy == gcn_server_tls_policy.ServerTlsPolicy(
            name="name_value"
        )
        assert args[0].server_tls_policy_id == "server_tls_policy_id_value"


@pytest.mark.asyncio
async def test_create_server_tls_policy_flattened_error_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.create_server_tls_policy(
            gcn_server_tls_policy.CreateServerTlsPolicyRequest(),
            parent="parent_value",
            server_tls_policy=gcn_server_tls_policy.ServerTlsPolicy(name="name_value"),
            server_tls_policy_id="server_tls_policy_id_value",
        )


def test_update_server_tls_policy(
    transport: str = "grpc",
    request_type=gcn_server_tls_policy.UpdateServerTlsPolicyRequest,
):
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_server_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/spam")
        response = client.update_server_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_server_tls_policy.UpdateServerTlsPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


def test_update_server_tls_policy_from_dict():
    test_update_server_tls_policy(request_type=dict)


def test_update_server_tls_policy_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_server_tls_policy), "__call__"
    ) as call:
        client.update_server_tls_policy()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_server_tls_policy.UpdateServerTlsPolicyRequest()


@pytest.mark.asyncio
async def test_update_server_tls_policy_async(
    transport: str = "grpc_asyncio",
    request_type=gcn_server_tls_policy.UpdateServerTlsPolicyRequest,
):
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_server_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        response = await client.update_server_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_server_tls_policy.UpdateServerTlsPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


@pytest.mark.asyncio
async def test_update_server_tls_policy_async_from_dict():
    await test_update_server_tls_policy_async(request_type=dict)


def test_update_server_tls_policy_field_headers():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = gcn_server_tls_policy.UpdateServerTlsPolicyRequest()

    request.server_tls_policy.name = "server_tls_policy.name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_server_tls_policy), "__call__"
    ) as call:
        call.return_value = operations_pb2.Operation(name="operations/op")
        client.update_server_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert (
        "x-goog-request-params",
        "server_tls_policy.name=server_tls_policy.name/value",
    ) in kw["metadata"]


@pytest.mark.asyncio
async def test_update_server_tls_policy_field_headers_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = gcn_server_tls_policy.UpdateServerTlsPolicyRequest()

    request.server_tls_policy.name = "server_tls_policy.name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_server_tls_policy), "__call__"
    ) as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/op")
        )
        await client.update_server_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert (
        "x-goog-request-params",
        "server_tls_policy.name=server_tls_policy.name/value",
    ) in kw["metadata"]


def test_update_server_tls_policy_flattened():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_server_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.update_server_tls_policy(
            server_tls_policy=gcn_server_tls_policy.ServerTlsPolicy(name="name_value"),
            update_mask=field_mask_pb2.FieldMask(paths=["paths_value"]),
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0].server_tls_policy == gcn_server_tls_policy.ServerTlsPolicy(
            name="name_value"
        )
        assert args[0].update_mask == field_mask_pb2.FieldMask(paths=["paths_value"])


def test_update_server_tls_policy_flattened_error():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.update_server_tls_policy(
            gcn_server_tls_policy.UpdateServerTlsPolicyRequest(),
            server_tls_policy=gcn_server_tls_policy.ServerTlsPolicy(name="name_value"),
            update_mask=field_mask_pb2.FieldMask(paths=["paths_value"]),
        )


@pytest.mark.asyncio
async def test_update_server_tls_policy_flattened_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_server_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.update_server_tls_policy(
            server_tls_policy=gcn_server_tls_policy.ServerTlsPolicy(name="name_value"),
            update_mask=field_mask_pb2.FieldMask(paths=["paths_value"]),
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0].server_tls_policy == gcn_server_tls_policy.ServerTlsPolicy(
            name="name_value"
        )
        assert args[0].update_mask == field_mask_pb2.FieldMask(paths=["paths_value"])


@pytest.mark.asyncio
async def test_update_server_tls_policy_flattened_error_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.update_server_tls_policy(
            gcn_server_tls_policy.UpdateServerTlsPolicyRequest(),
            server_tls_policy=gcn_server_tls_policy.ServerTlsPolicy(name="name_value"),
            update_mask=field_mask_pb2.FieldMask(paths=["paths_value"]),
        )


def test_delete_server_tls_policy(
    transport: str = "grpc", request_type=server_tls_policy.DeleteServerTlsPolicyRequest
):
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_server_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/spam")
        response = client.delete_server_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == server_tls_policy.DeleteServerTlsPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


def test_delete_server_tls_policy_from_dict():
    test_delete_server_tls_policy(request_type=dict)


def test_delete_server_tls_policy_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_server_tls_policy), "__call__"
    ) as call:
        client.delete_server_tls_policy()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == server_tls_policy.DeleteServerTlsPolicyRequest()


@pytest.mark.asyncio
async def test_delete_server_tls_policy_async(
    transport: str = "grpc_asyncio",
    request_type=server_tls_policy.DeleteServerTlsPolicyRequest,
):
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_server_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        response = await client.delete_server_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == server_tls_policy.DeleteServerTlsPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


@pytest.mark.asyncio
async def test_delete_server_tls_policy_async_from_dict():
    await test_delete_server_tls_policy_async(request_type=dict)


def test_delete_server_tls_policy_field_headers():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = server_tls_policy.DeleteServerTlsPolicyRequest()

    request.name = "name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_server_tls_policy), "__call__"
    ) as call:
        call.return_value = operations_pb2.Operation(name="operations/op")
        client.delete_server_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]


@pytest.mark.asyncio
async def test_delete_server_tls_policy_field_headers_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = server_tls_policy.DeleteServerTlsPolicyRequest()

    request.name = "name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_server_tls_policy), "__call__"
    ) as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/op")
        )
        await client.delete_server_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]


def test_delete_server_tls_policy_flattened():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_server_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.delete_server_tls_policy(name="name_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0].name == "name_value"


def test_delete_server_tls_policy_flattened_error():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.delete_server_tls_policy(
            server_tls_policy.DeleteServerTlsPolicyRequest(), name="name_value",
        )


@pytest.mark.asyncio
async def test_delete_server_tls_policy_flattened_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_server_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.delete_server_tls_policy(name="name_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0].name == "name_value"


@pytest.mark.asyncio
async def test_delete_server_tls_policy_flattened_error_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.delete_server_tls_policy(
            server_tls_policy.DeleteServerTlsPolicyRequest(), name="name_value",
        )


def test_list_client_tls_policies(
    transport: str = "grpc", request_type=client_tls_policy.ListClientTlsPoliciesRequest
):
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_client_tls_policies), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = client_tls_policy.ListClientTlsPoliciesResponse(
            next_page_token="next_page_token_value",
        )
        response = client.list_client_tls_policies(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == client_tls_policy.ListClientTlsPoliciesRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, pagers.ListClientTlsPoliciesPager)
    assert response.next_page_token == "next_page_token_value"


def test_list_client_tls_policies_from_dict():
    test_list_client_tls_policies(request_type=dict)


def test_list_client_tls_policies_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_client_tls_policies), "__call__"
    ) as call:
        client.list_client_tls_policies()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == client_tls_policy.ListClientTlsPoliciesRequest()


@pytest.mark.asyncio
async def test_list_client_tls_policies_async(
    transport: str = "grpc_asyncio",
    request_type=client_tls_policy.ListClientTlsPoliciesRequest,
):
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_client_tls_policies), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            client_tls_policy.ListClientTlsPoliciesResponse(
                next_page_token="next_page_token_value",
            )
        )
        response = await client.list_client_tls_policies(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == client_tls_policy.ListClientTlsPoliciesRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, pagers.ListClientTlsPoliciesAsyncPager)
    assert response.next_page_token == "next_page_token_value"


@pytest.mark.asyncio
async def test_list_client_tls_policies_async_from_dict():
    await test_list_client_tls_policies_async(request_type=dict)


def test_list_client_tls_policies_field_headers():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = client_tls_policy.ListClientTlsPoliciesRequest()

    request.parent = "parent/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_client_tls_policies), "__call__"
    ) as call:
        call.return_value = client_tls_policy.ListClientTlsPoliciesResponse()
        client.list_client_tls_policies(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "parent=parent/value",) in kw["metadata"]


@pytest.mark.asyncio
async def test_list_client_tls_policies_field_headers_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = client_tls_policy.ListClientTlsPoliciesRequest()

    request.parent = "parent/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_client_tls_policies), "__call__"
    ) as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            client_tls_policy.ListClientTlsPoliciesResponse()
        )
        await client.list_client_tls_policies(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "parent=parent/value",) in kw["metadata"]


def test_list_client_tls_policies_flattened():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_client_tls_policies), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = client_tls_policy.ListClientTlsPoliciesResponse()
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.list_client_tls_policies(parent="parent_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0].parent == "parent_value"


def test_list_client_tls_policies_flattened_error():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.list_client_tls_policies(
            client_tls_policy.ListClientTlsPoliciesRequest(), parent="parent_value",
        )


@pytest.mark.asyncio
async def test_list_client_tls_policies_flattened_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_client_tls_policies), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = client_tls_policy.ListClientTlsPoliciesResponse()

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            client_tls_policy.ListClientTlsPoliciesResponse()
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.list_client_tls_policies(parent="parent_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0].parent == "parent_value"


@pytest.mark.asyncio
async def test_list_client_tls_policies_flattened_error_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.list_client_tls_policies(
            client_tls_policy.ListClientTlsPoliciesRequest(), parent="parent_value",
        )


def test_list_client_tls_policies_pager():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials,)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_client_tls_policies), "__call__"
    ) as call:
        # Set the response to a series of pages.
        call.side_effect = (
            client_tls_policy.ListClientTlsPoliciesResponse(
                client_tls_policies=[
                    client_tls_policy.ClientTlsPolicy(),
                    client_tls_policy.ClientTlsPolicy(),
                    client_tls_policy.ClientTlsPolicy(),
                ],
                next_page_token="abc",
            ),
            client_tls_policy.ListClientTlsPoliciesResponse(
                client_tls_policies=[], next_page_token="def",
            ),
            client_tls_policy.ListClientTlsPoliciesResponse(
                client_tls_policies=[client_tls_policy.ClientTlsPolicy(),],
                next_page_token="ghi",
            ),
            client_tls_policy.ListClientTlsPoliciesResponse(
                client_tls_policies=[
                    client_tls_policy.ClientTlsPolicy(),
                    client_tls_policy.ClientTlsPolicy(),
                ],
            ),
            RuntimeError,
        )

        metadata = ()
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", ""),)),
        )
        pager = client.list_client_tls_policies(request={})

        assert pager._metadata == metadata

        results = [i for i in pager]
        assert len(results) == 6
        assert all(isinstance(i, client_tls_policy.ClientTlsPolicy) for i in results)


def test_list_client_tls_policies_pages():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials,)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_client_tls_policies), "__call__"
    ) as call:
        # Set the response to a series of pages.
        call.side_effect = (
            client_tls_policy.ListClientTlsPoliciesResponse(
                client_tls_policies=[
                    client_tls_policy.ClientTlsPolicy(),
                    client_tls_policy.ClientTlsPolicy(),
                    client_tls_policy.ClientTlsPolicy(),
                ],
                next_page_token="abc",
            ),
            client_tls_policy.ListClientTlsPoliciesResponse(
                client_tls_policies=[], next_page_token="def",
            ),
            client_tls_policy.ListClientTlsPoliciesResponse(
                client_tls_policies=[client_tls_policy.ClientTlsPolicy(),],
                next_page_token="ghi",
            ),
            client_tls_policy.ListClientTlsPoliciesResponse(
                client_tls_policies=[
                    client_tls_policy.ClientTlsPolicy(),
                    client_tls_policy.ClientTlsPolicy(),
                ],
            ),
            RuntimeError,
        )
        pages = list(client.list_client_tls_policies(request={}).pages)
        for page_, token in zip(pages, ["abc", "def", "ghi", ""]):
            assert page_.raw_page.next_page_token == token


@pytest.mark.asyncio
async def test_list_client_tls_policies_async_pager():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials,
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_client_tls_policies),
        "__call__",
        new_callable=mock.AsyncMock,
    ) as call:
        # Set the response to a series of pages.
        call.side_effect = (
            client_tls_policy.ListClientTlsPoliciesResponse(
                client_tls_policies=[
                    client_tls_policy.ClientTlsPolicy(),
                    client_tls_policy.ClientTlsPolicy(),
                    client_tls_policy.ClientTlsPolicy(),
                ],
                next_page_token="abc",
            ),
            client_tls_policy.ListClientTlsPoliciesResponse(
                client_tls_policies=[], next_page_token="def",
            ),
            client_tls_policy.ListClientTlsPoliciesResponse(
                client_tls_policies=[client_tls_policy.ClientTlsPolicy(),],
                next_page_token="ghi",
            ),
            client_tls_policy.ListClientTlsPoliciesResponse(
                client_tls_policies=[
                    client_tls_policy.ClientTlsPolicy(),
                    client_tls_policy.ClientTlsPolicy(),
                ],
            ),
            RuntimeError,
        )
        async_pager = await client.list_client_tls_policies(request={},)
        assert async_pager.next_page_token == "abc"
        responses = []
        async for response in async_pager:
            responses.append(response)

        assert len(responses) == 6
        assert all(isinstance(i, client_tls_policy.ClientTlsPolicy) for i in responses)


@pytest.mark.asyncio
async def test_list_client_tls_policies_async_pages():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials,
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.list_client_tls_policies),
        "__call__",
        new_callable=mock.AsyncMock,
    ) as call:
        # Set the response to a series of pages.
        call.side_effect = (
            client_tls_policy.ListClientTlsPoliciesResponse(
                client_tls_policies=[
                    client_tls_policy.ClientTlsPolicy(),
                    client_tls_policy.ClientTlsPolicy(),
                    client_tls_policy.ClientTlsPolicy(),
                ],
                next_page_token="abc",
            ),
            client_tls_policy.ListClientTlsPoliciesResponse(
                client_tls_policies=[], next_page_token="def",
            ),
            client_tls_policy.ListClientTlsPoliciesResponse(
                client_tls_policies=[client_tls_policy.ClientTlsPolicy(),],
                next_page_token="ghi",
            ),
            client_tls_policy.ListClientTlsPoliciesResponse(
                client_tls_policies=[
                    client_tls_policy.ClientTlsPolicy(),
                    client_tls_policy.ClientTlsPolicy(),
                ],
            ),
            RuntimeError,
        )
        pages = []
        async for page_ in (await client.list_client_tls_policies(request={})).pages:
            pages.append(page_)
        for page_, token in zip(pages, ["abc", "def", "ghi", ""]):
            assert page_.raw_page.next_page_token == token


def test_get_client_tls_policy(
    transport: str = "grpc", request_type=client_tls_policy.GetClientTlsPolicyRequest
):
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_client_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = client_tls_policy.ClientTlsPolicy(
            name="name_value", description="description_value", sni="sni_value",
        )
        response = client.get_client_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == client_tls_policy.GetClientTlsPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, client_tls_policy.ClientTlsPolicy)
    assert response.name == "name_value"
    assert response.description == "description_value"
    assert response.sni == "sni_value"


def test_get_client_tls_policy_from_dict():
    test_get_client_tls_policy(request_type=dict)


def test_get_client_tls_policy_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_client_tls_policy), "__call__"
    ) as call:
        client.get_client_tls_policy()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == client_tls_policy.GetClientTlsPolicyRequest()


@pytest.mark.asyncio
async def test_get_client_tls_policy_async(
    transport: str = "grpc_asyncio",
    request_type=client_tls_policy.GetClientTlsPolicyRequest,
):
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_client_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            client_tls_policy.ClientTlsPolicy(
                name="name_value", description="description_value", sni="sni_value",
            )
        )
        response = await client.get_client_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == client_tls_policy.GetClientTlsPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, client_tls_policy.ClientTlsPolicy)
    assert response.name == "name_value"
    assert response.description == "description_value"
    assert response.sni == "sni_value"


@pytest.mark.asyncio
async def test_get_client_tls_policy_async_from_dict():
    await test_get_client_tls_policy_async(request_type=dict)


def test_get_client_tls_policy_field_headers():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = client_tls_policy.GetClientTlsPolicyRequest()

    request.name = "name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_client_tls_policy), "__call__"
    ) as call:
        call.return_value = client_tls_policy.ClientTlsPolicy()
        client.get_client_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]


@pytest.mark.asyncio
async def test_get_client_tls_policy_field_headers_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = client_tls_policy.GetClientTlsPolicyRequest()

    request.name = "name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_client_tls_policy), "__call__"
    ) as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            client_tls_policy.ClientTlsPolicy()
        )
        await client.get_client_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]


def test_get_client_tls_policy_flattened():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_client_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = client_tls_policy.ClientTlsPolicy()
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.get_client_tls_policy(name="name_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0].name == "name_value"


def test_get_client_tls_policy_flattened_error():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.get_client_tls_policy(
            client_tls_policy.GetClientTlsPolicyRequest(), name="name_value",
        )


@pytest.mark.asyncio
async def test_get_client_tls_policy_flattened_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.get_client_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = client_tls_policy.ClientTlsPolicy()

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            client_tls_policy.ClientTlsPolicy()
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.get_client_tls_policy(name="name_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0].name == "name_value"


@pytest.mark.asyncio
async def test_get_client_tls_policy_flattened_error_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.get_client_tls_policy(
            client_tls_policy.GetClientTlsPolicyRequest(), name="name_value",
        )


def test_create_client_tls_policy(
    transport: str = "grpc",
    request_type=gcn_client_tls_policy.CreateClientTlsPolicyRequest,
):
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_client_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/spam")
        response = client.create_client_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_client_tls_policy.CreateClientTlsPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


def test_create_client_tls_policy_from_dict():
    test_create_client_tls_policy(request_type=dict)


def test_create_client_tls_policy_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_client_tls_policy), "__call__"
    ) as call:
        client.create_client_tls_policy()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_client_tls_policy.CreateClientTlsPolicyRequest()


@pytest.mark.asyncio
async def test_create_client_tls_policy_async(
    transport: str = "grpc_asyncio",
    request_type=gcn_client_tls_policy.CreateClientTlsPolicyRequest,
):
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_client_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        response = await client.create_client_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_client_tls_policy.CreateClientTlsPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


@pytest.mark.asyncio
async def test_create_client_tls_policy_async_from_dict():
    await test_create_client_tls_policy_async(request_type=dict)


def test_create_client_tls_policy_field_headers():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = gcn_client_tls_policy.CreateClientTlsPolicyRequest()

    request.parent = "parent/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_client_tls_policy), "__call__"
    ) as call:
        call.return_value = operations_pb2.Operation(name="operations/op")
        client.create_client_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "parent=parent/value",) in kw["metadata"]


@pytest.mark.asyncio
async def test_create_client_tls_policy_field_headers_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = gcn_client_tls_policy.CreateClientTlsPolicyRequest()

    request.parent = "parent/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_client_tls_policy), "__call__"
    ) as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/op")
        )
        await client.create_client_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "parent=parent/value",) in kw["metadata"]


def test_create_client_tls_policy_flattened():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_client_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.create_client_tls_policy(
            parent="parent_value",
            client_tls_policy=gcn_client_tls_policy.ClientTlsPolicy(name="name_value"),
            client_tls_policy_id="client_tls_policy_id_value",
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0].parent == "parent_value"
        assert args[0].client_tls_policy == gcn_client_tls_policy.ClientTlsPolicy(
            name="name_value"
        )
        assert args[0].client_tls_policy_id == "client_tls_policy_id_value"


def test_create_client_tls_policy_flattened_error():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.create_client_tls_policy(
            gcn_client_tls_policy.CreateClientTlsPolicyRequest(),
            parent="parent_value",
            client_tls_policy=gcn_client_tls_policy.ClientTlsPolicy(name="name_value"),
            client_tls_policy_id="client_tls_policy_id_value",
        )


@pytest.mark.asyncio
async def test_create_client_tls_policy_flattened_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.create_client_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.create_client_tls_policy(
            parent="parent_value",
            client_tls_policy=gcn_client_tls_policy.ClientTlsPolicy(name="name_value"),
            client_tls_policy_id="client_tls_policy_id_value",
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0].parent == "parent_value"
        assert args[0].client_tls_policy == gcn_client_tls_policy.ClientTlsPolicy(
            name="name_value"
        )
        assert args[0].client_tls_policy_id == "client_tls_policy_id_value"


@pytest.mark.asyncio
async def test_create_client_tls_policy_flattened_error_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.create_client_tls_policy(
            gcn_client_tls_policy.CreateClientTlsPolicyRequest(),
            parent="parent_value",
            client_tls_policy=gcn_client_tls_policy.ClientTlsPolicy(name="name_value"),
            client_tls_policy_id="client_tls_policy_id_value",
        )


def test_update_client_tls_policy(
    transport: str = "grpc",
    request_type=gcn_client_tls_policy.UpdateClientTlsPolicyRequest,
):
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_client_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/spam")
        response = client.update_client_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_client_tls_policy.UpdateClientTlsPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


def test_update_client_tls_policy_from_dict():
    test_update_client_tls_policy(request_type=dict)


def test_update_client_tls_policy_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_client_tls_policy), "__call__"
    ) as call:
        client.update_client_tls_policy()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_client_tls_policy.UpdateClientTlsPolicyRequest()


@pytest.mark.asyncio
async def test_update_client_tls_policy_async(
    transport: str = "grpc_asyncio",
    request_type=gcn_client_tls_policy.UpdateClientTlsPolicyRequest,
):
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_client_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        response = await client.update_client_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == gcn_client_tls_policy.UpdateClientTlsPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


@pytest.mark.asyncio
async def test_update_client_tls_policy_async_from_dict():
    await test_update_client_tls_policy_async(request_type=dict)


def test_update_client_tls_policy_field_headers():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = gcn_client_tls_policy.UpdateClientTlsPolicyRequest()

    request.client_tls_policy.name = "client_tls_policy.name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_client_tls_policy), "__call__"
    ) as call:
        call.return_value = operations_pb2.Operation(name="operations/op")
        client.update_client_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert (
        "x-goog-request-params",
        "client_tls_policy.name=client_tls_policy.name/value",
    ) in kw["metadata"]


@pytest.mark.asyncio
async def test_update_client_tls_policy_field_headers_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = gcn_client_tls_policy.UpdateClientTlsPolicyRequest()

    request.client_tls_policy.name = "client_tls_policy.name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_client_tls_policy), "__call__"
    ) as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/op")
        )
        await client.update_client_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert (
        "x-goog-request-params",
        "client_tls_policy.name=client_tls_policy.name/value",
    ) in kw["metadata"]


def test_update_client_tls_policy_flattened():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_client_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.update_client_tls_policy(
            client_tls_policy=gcn_client_tls_policy.ClientTlsPolicy(name="name_value"),
            update_mask=field_mask_pb2.FieldMask(paths=["paths_value"]),
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0].client_tls_policy == gcn_client_tls_policy.ClientTlsPolicy(
            name="name_value"
        )
        assert args[0].update_mask == field_mask_pb2.FieldMask(paths=["paths_value"])


def test_update_client_tls_policy_flattened_error():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.update_client_tls_policy(
            gcn_client_tls_policy.UpdateClientTlsPolicyRequest(),
            client_tls_policy=gcn_client_tls_policy.ClientTlsPolicy(name="name_value"),
            update_mask=field_mask_pb2.FieldMask(paths=["paths_value"]),
        )


@pytest.mark.asyncio
async def test_update_client_tls_policy_flattened_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.update_client_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.update_client_tls_policy(
            client_tls_policy=gcn_client_tls_policy.ClientTlsPolicy(name="name_value"),
            update_mask=field_mask_pb2.FieldMask(paths=["paths_value"]),
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0].client_tls_policy == gcn_client_tls_policy.ClientTlsPolicy(
            name="name_value"
        )
        assert args[0].update_mask == field_mask_pb2.FieldMask(paths=["paths_value"])


@pytest.mark.asyncio
async def test_update_client_tls_policy_flattened_error_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.update_client_tls_policy(
            gcn_client_tls_policy.UpdateClientTlsPolicyRequest(),
            client_tls_policy=gcn_client_tls_policy.ClientTlsPolicy(name="name_value"),
            update_mask=field_mask_pb2.FieldMask(paths=["paths_value"]),
        )


def test_delete_client_tls_policy(
    transport: str = "grpc", request_type=client_tls_policy.DeleteClientTlsPolicyRequest
):
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_client_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/spam")
        response = client.delete_client_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == client_tls_policy.DeleteClientTlsPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


def test_delete_client_tls_policy_from_dict():
    test_delete_client_tls_policy(request_type=dict)


def test_delete_client_tls_policy_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_client_tls_policy), "__call__"
    ) as call:
        client.delete_client_tls_policy()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == client_tls_policy.DeleteClientTlsPolicyRequest()


@pytest.mark.asyncio
async def test_delete_client_tls_policy_async(
    transport: str = "grpc_asyncio",
    request_type=client_tls_policy.DeleteClientTlsPolicyRequest,
):
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_client_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        response = await client.delete_client_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == client_tls_policy.DeleteClientTlsPolicyRequest()

    # Establish that the response is the type that we expect.
    assert isinstance(response, future.Future)


@pytest.mark.asyncio
async def test_delete_client_tls_policy_async_from_dict():
    await test_delete_client_tls_policy_async(request_type=dict)


def test_delete_client_tls_policy_field_headers():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = client_tls_policy.DeleteClientTlsPolicyRequest()

    request.name = "name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_client_tls_policy), "__call__"
    ) as call:
        call.return_value = operations_pb2.Operation(name="operations/op")
        client.delete_client_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]


@pytest.mark.asyncio
async def test_delete_client_tls_policy_field_headers_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = client_tls_policy.DeleteClientTlsPolicyRequest()

    request.name = "name/value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_client_tls_policy), "__call__"
    ) as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/op")
        )
        await client.delete_client_tls_policy(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]


def test_delete_client_tls_policy_flattened():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_client_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.delete_client_tls_policy(name="name_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0].name == "name_value"


def test_delete_client_tls_policy_flattened_error():
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.delete_client_tls_policy(
            client_tls_policy.DeleteClientTlsPolicyRequest(), name="name_value",
        )


@pytest.mark.asyncio
async def test_delete_client_tls_policy_flattened_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.delete_client_tls_policy), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = operations_pb2.Operation(name="operations/op")

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            operations_pb2.Operation(name="operations/spam")
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.delete_client_tls_policy(name="name_value",)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0].name == "name_value"


@pytest.mark.asyncio
async def test_delete_client_tls_policy_flattened_error_async():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.delete_client_tls_policy(
            client_tls_policy.DeleteClientTlsPolicyRequest(), name="name_value",
        )


def test_credentials_transport_error():
    # It is an error to provide credentials and a transport instance.
    transport = transports.NetworkSecurityGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    with pytest.raises(ValueError):
        client = NetworkSecurityClient(
            credentials=ga_credentials.AnonymousCredentials(), transport=transport,
        )

    # It is an error to provide a credentials file and a transport instance.
    transport = transports.NetworkSecurityGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    with pytest.raises(ValueError):
        client = NetworkSecurityClient(
            client_options={"credentials_file": "credentials.json"},
            transport=transport,
        )

    # It is an error to provide scopes and a transport instance.
    transport = transports.NetworkSecurityGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    with pytest.raises(ValueError):
        client = NetworkSecurityClient(
            client_options={"scopes": ["1", "2"]}, transport=transport,
        )


def test_transport_instance():
    # A client may be instantiated with a custom transport instance.
    transport = transports.NetworkSecurityGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    client = NetworkSecurityClient(transport=transport)
    assert client.transport is transport


def test_transport_get_channel():
    # A client may be instantiated with a custom transport instance.
    transport = transports.NetworkSecurityGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    channel = transport.grpc_channel
    assert channel

    transport = transports.NetworkSecurityGrpcAsyncIOTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    channel = transport.grpc_channel
    assert channel


@pytest.mark.parametrize(
    "transport_class",
    [
        transports.NetworkSecurityGrpcTransport,
        transports.NetworkSecurityGrpcAsyncIOTransport,
    ],
)
def test_transport_adc(transport_class):
    # Test default credentials are used if not provided.
    with mock.patch.object(google.auth, "default") as adc:
        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
        transport_class()
        adc.assert_called_once()


def test_transport_grpc_default():
    # A client should use the gRPC transport by default.
    client = NetworkSecurityClient(credentials=ga_credentials.AnonymousCredentials(),)
    assert isinstance(client.transport, transports.NetworkSecurityGrpcTransport,)


def test_network_security_base_transport_error():
    # Passing both a credentials object and credentials_file should raise an error
    with pytest.raises(core_exceptions.DuplicateCredentialArgs):
        transport = transports.NetworkSecurityTransport(
            credentials=ga_credentials.AnonymousCredentials(),
            credentials_file="credentials.json",
        )


def test_network_security_base_transport():
    # Instantiate the base transport.
    with mock.patch(
        "google.cloud.network_security_v1beta1.services.network_security.transports.NetworkSecurityTransport.__init__"
    ) as Transport:
        Transport.return_value = None
        transport = transports.NetworkSecurityTransport(
            credentials=ga_credentials.AnonymousCredentials(),
        )

    # Every method on the transport should just blindly
    # raise NotImplementedError.
    methods = (
        "list_authorization_policies",
        "get_authorization_policy",
        "create_authorization_policy",
        "update_authorization_policy",
        "delete_authorization_policy",
        "list_server_tls_policies",
        "get_server_tls_policy",
        "create_server_tls_policy",
        "update_server_tls_policy",
        "delete_server_tls_policy",
        "list_client_tls_policies",
        "get_client_tls_policy",
        "create_client_tls_policy",
        "update_client_tls_policy",
        "delete_client_tls_policy",
    )
    for method in methods:
        with pytest.raises(NotImplementedError):
            getattr(transport, method)(request=object())

    # Additionally, the LRO client (a property) should
    # also raise NotImplementedError
    with pytest.raises(NotImplementedError):
        transport.operations_client


@requires_google_auth_gte_1_25_0
def test_network_security_base_transport_with_credentials_file():
    # Instantiate the base transport with a credentials file
    with mock.patch.object(
        google.auth, "load_credentials_from_file", autospec=True
    ) as load_creds, mock.patch(
        "google.cloud.network_security_v1beta1.services.network_security.transports.NetworkSecurityTransport._prep_wrapped_messages"
    ) as Transport:
        Transport.return_value = None
        load_creds.return_value = (ga_credentials.AnonymousCredentials(), None)
        transport = transports.NetworkSecurityTransport(
            credentials_file="credentials.json", quota_project_id="octopus",
        )
        load_creds.assert_called_once_with(
            "credentials.json",
            scopes=None,
            default_scopes=("https://www.googleapis.com/auth/cloud-platform",),
            quota_project_id="octopus",
        )


@requires_google_auth_lt_1_25_0
def test_network_security_base_transport_with_credentials_file_old_google_auth():
    # Instantiate the base transport with a credentials file
    with mock.patch.object(
        google.auth, "load_credentials_from_file", autospec=True
    ) as load_creds, mock.patch(
        "google.cloud.network_security_v1beta1.services.network_security.transports.NetworkSecurityTransport._prep_wrapped_messages"
    ) as Transport:
        Transport.return_value = None
        load_creds.return_value = (ga_credentials.AnonymousCredentials(), None)
        transport = transports.NetworkSecurityTransport(
            credentials_file="credentials.json", quota_project_id="octopus",
        )
        load_creds.assert_called_once_with(
            "credentials.json",
            scopes=("https://www.googleapis.com/auth/cloud-platform",),
            quota_project_id="octopus",
        )


def test_network_security_base_transport_with_adc():
    # Test the default credentials are used if credentials and credentials_file are None.
    with mock.patch.object(google.auth, "default", autospec=True) as adc, mock.patch(
        "google.cloud.network_security_v1beta1.services.network_security.transports.NetworkSecurityTransport._prep_wrapped_messages"
    ) as Transport:
        Transport.return_value = None
        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
        transport = transports.NetworkSecurityTransport()
        adc.assert_called_once()


@requires_google_auth_gte_1_25_0
def test_network_security_auth_adc():
    # If no credentials are provided, we should use ADC credentials.
    with mock.patch.object(google.auth, "default", autospec=True) as adc:
        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
        NetworkSecurityClient()
        adc.assert_called_once_with(
            scopes=None,
            default_scopes=("https://www.googleapis.com/auth/cloud-platform",),
            quota_project_id=None,
        )


@requires_google_auth_lt_1_25_0
def test_network_security_auth_adc_old_google_auth():
    # If no credentials are provided, we should use ADC credentials.
    with mock.patch.object(google.auth, "default", autospec=True) as adc:
        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
        NetworkSecurityClient()
        adc.assert_called_once_with(
            scopes=("https://www.googleapis.com/auth/cloud-platform",),
            quota_project_id=None,
        )


@pytest.mark.parametrize(
    "transport_class",
    [
        transports.NetworkSecurityGrpcTransport,
        transports.NetworkSecurityGrpcAsyncIOTransport,
    ],
)
@requires_google_auth_gte_1_25_0
def test_network_security_transport_auth_adc(transport_class):
    # If credentials and host are not provided, the transport class should use
    # ADC credentials.
    with mock.patch.object(google.auth, "default", autospec=True) as adc:
        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
        transport_class(quota_project_id="octopus", scopes=["1", "2"])
        adc.assert_called_once_with(
            scopes=["1", "2"],
            default_scopes=("https://www.googleapis.com/auth/cloud-platform",),
            quota_project_id="octopus",
        )


@pytest.mark.parametrize(
    "transport_class",
    [
        transports.NetworkSecurityGrpcTransport,
        transports.NetworkSecurityGrpcAsyncIOTransport,
    ],
)
@requires_google_auth_lt_1_25_0
def test_network_security_transport_auth_adc_old_google_auth(transport_class):
    # If credentials and host are not provided, the transport class should use
    # ADC credentials.
    with mock.patch.object(google.auth, "default", autospec=True) as adc:
        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
        transport_class(quota_project_id="octopus")
        adc.assert_called_once_with(
            scopes=("https://www.googleapis.com/auth/cloud-platform",),
            quota_project_id="octopus",
        )


@pytest.mark.parametrize(
    "transport_class,grpc_helpers",
    [
        (transports.NetworkSecurityGrpcTransport, grpc_helpers),
        (transports.NetworkSecurityGrpcAsyncIOTransport, grpc_helpers_async),
    ],
)
def test_network_security_transport_create_channel(transport_class, grpc_helpers):
    # If credentials and host are not provided, the transport class should use
    # ADC credentials.
    with mock.patch.object(
        google.auth, "default", autospec=True
    ) as adc, mock.patch.object(
        grpc_helpers, "create_channel", autospec=True
    ) as create_channel:
        creds = ga_credentials.AnonymousCredentials()
        adc.return_value = (creds, None)
        transport_class(quota_project_id="octopus", scopes=["1", "2"])

        create_channel.assert_called_with(
            "networksecurity.googleapis.com:443",
            credentials=creds,
            credentials_file=None,
            quota_project_id="octopus",
            default_scopes=("https://www.googleapis.com/auth/cloud-platform",),
            scopes=["1", "2"],
            default_host="networksecurity.googleapis.com",
            ssl_credentials=None,
            options=[
                ("grpc.max_send_message_length", -1),
                ("grpc.max_receive_message_length", -1),
            ],
        )


@pytest.mark.parametrize(
    "transport_class",
    [
        transports.NetworkSecurityGrpcTransport,
        transports.NetworkSecurityGrpcAsyncIOTransport,
    ],
)
def test_network_security_grpc_transport_client_cert_source_for_mtls(transport_class):
    cred = ga_credentials.AnonymousCredentials()

    # Check ssl_channel_credentials is used if provided.
    with mock.patch.object(transport_class, "create_channel") as mock_create_channel:
        mock_ssl_channel_creds = mock.Mock()
        transport_class(
            host="squid.clam.whelk",
            credentials=cred,
            ssl_channel_credentials=mock_ssl_channel_creds,
        )
        mock_create_channel.assert_called_once_with(
            "squid.clam.whelk:443",
            credentials=cred,
            credentials_file=None,
            scopes=None,
            ssl_credentials=mock_ssl_channel_creds,
            quota_project_id=None,
            options=[
                ("grpc.max_send_message_length", -1),
                ("grpc.max_receive_message_length", -1),
            ],
        )

    # Check if ssl_channel_credentials is not provided, then client_cert_source_for_mtls
    # is used.
    with mock.patch.object(transport_class, "create_channel", return_value=mock.Mock()):
        with mock.patch("grpc.ssl_channel_credentials") as mock_ssl_cred:
            transport_class(
                credentials=cred,
                client_cert_source_for_mtls=client_cert_source_callback,
            )
            expected_cert, expected_key = client_cert_source_callback()
            mock_ssl_cred.assert_called_once_with(
                certificate_chain=expected_cert, private_key=expected_key
            )


def test_network_security_host_no_port():
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(),
        client_options=client_options.ClientOptions(
            api_endpoint="networksecurity.googleapis.com"
        ),
    )
    assert client.transport._host == "networksecurity.googleapis.com:443"


def test_network_security_host_with_port():
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(),
        client_options=client_options.ClientOptions(
            api_endpoint="networksecurity.googleapis.com:8000"
        ),
    )
    assert client.transport._host == "networksecurity.googleapis.com:8000"


def test_network_security_grpc_transport_channel():
    channel = grpc.secure_channel("http://localhost/", grpc.local_channel_credentials())

    # Check that channel is used if provided.
    transport = transports.NetworkSecurityGrpcTransport(
        host="squid.clam.whelk", channel=channel,
    )
    assert transport.grpc_channel == channel
    assert transport._host == "squid.clam.whelk:443"
    assert transport._ssl_channel_credentials == None


def test_network_security_grpc_asyncio_transport_channel():
    channel = aio.secure_channel("http://localhost/", grpc.local_channel_credentials())

    # Check that channel is used if provided.
    transport = transports.NetworkSecurityGrpcAsyncIOTransport(
        host="squid.clam.whelk", channel=channel,
    )
    assert transport.grpc_channel == channel
    assert transport._host == "squid.clam.whelk:443"
    assert transport._ssl_channel_credentials == None


# Remove this test when deprecated arguments (api_mtls_endpoint, client_cert_source) are
# removed from grpc/grpc_asyncio transport constructor.
@pytest.mark.parametrize(
    "transport_class",
    [
        transports.NetworkSecurityGrpcTransport,
        transports.NetworkSecurityGrpcAsyncIOTransport,
    ],
)
def test_network_security_transport_channel_mtls_with_client_cert_source(
    transport_class,
):
    with mock.patch(
        "grpc.ssl_channel_credentials", autospec=True
    ) as grpc_ssl_channel_cred:
        with mock.patch.object(
            transport_class, "create_channel"
        ) as grpc_create_channel:
            mock_ssl_cred = mock.Mock()
            grpc_ssl_channel_cred.return_value = mock_ssl_cred

            mock_grpc_channel = mock.Mock()
            grpc_create_channel.return_value = mock_grpc_channel

            cred = ga_credentials.AnonymousCredentials()
            with pytest.warns(DeprecationWarning):
                with mock.patch.object(google.auth, "default") as adc:
                    adc.return_value = (cred, None)
                    transport = transport_class(
                        host="squid.clam.whelk",
                        api_mtls_endpoint="mtls.squid.clam.whelk",
                        client_cert_source=client_cert_source_callback,
                    )
                    adc.assert_called_once()

            grpc_ssl_channel_cred.assert_called_once_with(
                certificate_chain=b"cert bytes", private_key=b"key bytes"
            )
            grpc_create_channel.assert_called_once_with(
                "mtls.squid.clam.whelk:443",
                credentials=cred,
                credentials_file=None,
                scopes=None,
                ssl_credentials=mock_ssl_cred,
                quota_project_id=None,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )
            assert transport.grpc_channel == mock_grpc_channel
            assert transport._ssl_channel_credentials == mock_ssl_cred


# Remove this test when deprecated arguments (api_mtls_endpoint, client_cert_source) are
# removed from grpc/grpc_asyncio transport constructor.
@pytest.mark.parametrize(
    "transport_class",
    [
        transports.NetworkSecurityGrpcTransport,
        transports.NetworkSecurityGrpcAsyncIOTransport,
    ],
)
def test_network_security_transport_channel_mtls_with_adc(transport_class):
    mock_ssl_cred = mock.Mock()
    with mock.patch.multiple(
        "google.auth.transport.grpc.SslCredentials",
        __init__=mock.Mock(return_value=None),
        ssl_credentials=mock.PropertyMock(return_value=mock_ssl_cred),
    ):
        with mock.patch.object(
            transport_class, "create_channel"
        ) as grpc_create_channel:
            mock_grpc_channel = mock.Mock()
            grpc_create_channel.return_value = mock_grpc_channel
            mock_cred = mock.Mock()

            with pytest.warns(DeprecationWarning):
                transport = transport_class(
                    host="squid.clam.whelk",
                    credentials=mock_cred,
                    api_mtls_endpoint="mtls.squid.clam.whelk",
                    client_cert_source=None,
                )

            grpc_create_channel.assert_called_once_with(
                "mtls.squid.clam.whelk:443",
                credentials=mock_cred,
                credentials_file=None,
                scopes=None,
                ssl_credentials=mock_ssl_cred,
                quota_project_id=None,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )
            assert transport.grpc_channel == mock_grpc_channel


def test_network_security_grpc_lro_client():
    client = NetworkSecurityClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
    )
    transport = client.transport

    # Ensure that we have a api-core operations client.
    assert isinstance(transport.operations_client, operations_v1.OperationsClient,)

    # Ensure that subsequent calls to the property send the exact same object.
    assert transport.operations_client is transport.operations_client


def test_network_security_grpc_lro_async_client():
    client = NetworkSecurityAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc_asyncio",
    )
    transport = client.transport

    # Ensure that we have a api-core operations client.
    assert isinstance(transport.operations_client, operations_v1.OperationsAsyncClient,)

    # Ensure that subsequent calls to the property send the exact same object.
    assert transport.operations_client is transport.operations_client


def test_authorization_policy_path():
    project = "squid"
    location = "clam"
    authorization_policy = "whelk"
    expected = "projects/{project}/locations/{location}/authorizationPolicies/{authorization_policy}".format(
        project=project, location=location, authorization_policy=authorization_policy,
    )
    actual = NetworkSecurityClient.authorization_policy_path(
        project, location, authorization_policy
    )
    assert expected == actual


def test_parse_authorization_policy_path():
    expected = {
        "project": "octopus",
        "location": "oyster",
        "authorization_policy": "nudibranch",
    }
    path = NetworkSecurityClient.authorization_policy_path(**expected)

    # Check that the path construction is reversible.
    actual = NetworkSecurityClient.parse_authorization_policy_path(path)
    assert expected == actual


def test_client_tls_policy_path():
    project = "cuttlefish"
    location = "mussel"
    client_tls_policy = "winkle"
    expected = "projects/{project}/locations/{location}/clientTlsPolicies/{client_tls_policy}".format(
        project=project, location=location, client_tls_policy=client_tls_policy,
    )
    actual = NetworkSecurityClient.client_tls_policy_path(
        project, location, client_tls_policy
    )
    assert expected == actual


def test_parse_client_tls_policy_path():
    expected = {
        "project": "nautilus",
        "location": "scallop",
        "client_tls_policy": "abalone",
    }
    path = NetworkSecurityClient.client_tls_policy_path(**expected)

    # Check that the path construction is reversible.
    actual = NetworkSecurityClient.parse_client_tls_policy_path(path)
    assert expected == actual


def test_server_tls_policy_path():
    project = "squid"
    location = "clam"
    server_tls_policy = "whelk"
    expected = "projects/{project}/locations/{location}/serverTlsPolicies/{server_tls_policy}".format(
        project=project, location=location, server_tls_policy=server_tls_policy,
    )
    actual = NetworkSecurityClient.server_tls_policy_path(
        project, location, server_tls_policy
    )
    assert expected == actual


def test_parse_server_tls_policy_path():
    expected = {
        "project": "octopus",
        "location": "oyster",
        "server_tls_policy": "nudibranch",
    }
    path = NetworkSecurityClient.server_tls_policy_path(**expected)

    # Check that the path construction is reversible.
    actual = NetworkSecurityClient.parse_server_tls_policy_path(path)
    assert expected == actual


def test_common_billing_account_path():
    billing_account = "cuttlefish"
    expected = "billingAccounts/{billing_account}".format(
        billing_account=billing_account,
    )
    actual = NetworkSecurityClient.common_billing_account_path(billing_account)
    assert expected == actual


def test_parse_common_billing_account_path():
    expected = {
        "billing_account": "mussel",
    }
    path = NetworkSecurityClient.common_billing_account_path(**expected)

    # Check that the path construction is reversible.
    actual = NetworkSecurityClient.parse_common_billing_account_path(path)
    assert expected == actual


def test_common_folder_path():
    folder = "winkle"
    expected = "folders/{folder}".format(folder=folder,)
    actual = NetworkSecurityClient.common_folder_path(folder)
    assert expected == actual


def test_parse_common_folder_path():
    expected = {
        "folder": "nautilus",
    }
    path = NetworkSecurityClient.common_folder_path(**expected)

    # Check that the path construction is reversible.
    actual = NetworkSecurityClient.parse_common_folder_path(path)
    assert expected == actual


def test_common_organization_path():
    organization = "scallop"
    expected = "organizations/{organization}".format(organization=organization,)
    actual = NetworkSecurityClient.common_organization_path(organization)
    assert expected == actual


def test_parse_common_organization_path():
    expected = {
        "organization": "abalone",
    }
    path = NetworkSecurityClient.common_organization_path(**expected)

    # Check that the path construction is reversible.
    actual = NetworkSecurityClient.parse_common_organization_path(path)
    assert expected == actual


def test_common_project_path():
    project = "squid"
    expected = "projects/{project}".format(project=project,)
    actual = NetworkSecurityClient.common_project_path(project)
    assert expected == actual


def test_parse_common_project_path():
    expected = {
        "project": "clam",
    }
    path = NetworkSecurityClient.common_project_path(**expected)

    # Check that the path construction is reversible.
    actual = NetworkSecurityClient.parse_common_project_path(path)
    assert expected == actual


def test_common_location_path():
    project = "whelk"
    location = "octopus"
    expected = "projects/{project}/locations/{location}".format(
        project=project, location=location,
    )
    actual = NetworkSecurityClient.common_location_path(project, location)
    assert expected == actual


def test_parse_common_location_path():
    expected = {
        "project": "oyster",
        "location": "nudibranch",
    }
    path = NetworkSecurityClient.common_location_path(**expected)

    # Check that the path construction is reversible.
    actual = NetworkSecurityClient.parse_common_location_path(path)
    assert expected == actual


def test_client_withDEFAULT_CLIENT_INFO():
    client_info = gapic_v1.client_info.ClientInfo()

    with mock.patch.object(
        transports.NetworkSecurityTransport, "_prep_wrapped_messages"
    ) as prep:
        client = NetworkSecurityClient(
            credentials=ga_credentials.AnonymousCredentials(), client_info=client_info,
        )
        prep.assert_called_once_with(client_info)

    with mock.patch.object(
        transports.NetworkSecurityTransport, "_prep_wrapped_messages"
    ) as prep:
        transport_class = NetworkSecurityClient.get_transport_class()
        transport = transport_class(
            credentials=ga_credentials.AnonymousCredentials(), client_info=client_info,
        )
        prep.assert_called_once_with(client_info)
